
import java.util.Scanner;

public class SmithWaterman {

    static double w(char a, char b) {
        if (a == '-' || b == '-') {
            return -0.6;
        } else if (a == b) {
            return 2;
        } else {
            return -1;
        }
    }

    static double max(double u, double l, double d) {
        if (d > u && d > l) {
            return d;
        } else if (u > l) {
            return u;
        } else {
            return l;
        }
    }

    static char dir(double u, double l, double d) {
        if (d > u && d > l) {
            return 'D';
        } else if (u > l) {
            return 'U';
        } else {
            return 'L';
        }
    }

    static String traceback(char D[][], int n, int m) {
        String res = "";
        if (n == 0 && m == 0) {
            return "";
        }
        if (D[n][m] == 'D') {
            res = traceback(D, n - 1, m - 1);
        }
        if (D[n][m] == 'L') {
            res = traceback(D, n, m - 1);
        }
        if (D[n][m] == 'U') {
            res = traceback(D, n - 1, m);
        }

        return res + D[n][m];
    }

    public static void main(String[] args) {
        String seq1 = "ACATACTA";
        String seq2 = "AGCACACA";

        int n = seq1.length();
        int m = seq2.length();

        char S1[] = new char[n + 1]; // sekwencje numerować będziemy od 1
        char S2[] = new char[m + 1];

        double H[][] = new double[n + 1][m + 1];
        char D[][] = new char[n + 1][m + 1];

        H[0][0] = 0;

        for (int i = 1; i <= n; i++) {
            S1[i] = seq1.charAt(i - 1);
            H[i][0] = 0;
            D[i][0] = 'U';
        }

        for (int j = 1; j <= m; j++) {
            S2[j] = seq2.charAt(j - 1);
            H[0][j] = 0;
            D[0][j] = 'L';
        }

        for (int i = 1; i <= n; i++) {
            for (int j = 1; j <= m; j++) {

                double up = H[i - 1][j] + w(S1[i], '-');
                double left = H[i][j - 1] + w('-', S2[j]);
                double diag = H[i - 1][j - 1] + w(S1[i], S2[j]);

                H[i][j] = max(up, left, diag);
                D[i][j] = dir(up, left, diag);
            }
        }

        String trace = traceback(D, n, m);

        String seqal1 = "";
        String seqal2 = "";

        int pos1 = 1;
        int pos2 = 1;

        for (int i = 0; i < trace.length(); i++) {
            switch (trace.charAt(i)) {
                case 'U':
                    seqal1 += S1[pos1++];
                    seqal2 += '-';

                    break;
                case 'L':
                    seqal1 += '-';
                    seqal2 += S2[pos2++];
                    break;
                case 'D':
                    seqal1 += S1[pos1++];
                    seqal2 += S2[pos2++];
                    break;

            }
        }

        System.out.println("Wynik:" + H[n][m]);
        System.out.println(seqal1);
        System.out.println(seqal2);

    }
}
